#ifndef _EXAMPLE_MLE_
#define _EXAMPLE_MLE_

#include <mpi.h>
#include <ScalaGAUSS.hpp>
extern "C" {
#include <petscsnes.h>
}


//-------------------------------------------------------------------------
//----- Global functions
// Functions required by the SNES solver of PETSC
PetscErrorCode FormInitialGuess(Vec, double*, int);
PetscErrorCode FormFunction(SNES, Vec, Vec, void*);
PetscErrorCode FormJacobian(SNES, Vec, Mat*, Mat*, MatStructure*, void*);
PetscErrorCode Monitor(SNES, PetscInt, PetscReal, void*);
// Load GP sample from file
void ReadData(char *filename, RegularGrid &grid, double *data, MPI_Comm &comm);
// Print timing
void PrintTimeStat(double t1, double t2, const char *header, MPI_Comm &comm);
// Print solution summary
void PrintSolSummary(int len_theta, Vec estim,
                     double **fisher, double **var_mle,
                     double **sandwich, double **var_mle_saa,
                     MPI_Comm &comm);


//-------------------------------------------------------------------------
// A class that wraps all the parameters needed by MLE
#define NDIM 3
#define MAX_FLEN 1000

class MLEParam {

public:

  MLEParam(MPI_Comm &_Comm);
  // The following function uses PETSC routines to maintain options database
  PetscErrorCode GetFromOptions(void);
  void PrintParams(void);

  int NDim;                     // Grid: number of dimensions
  long Dim[NDIM];               // Grid: size of each dimension
  GRID_TYPE GridType;           // Grid: type
  double Range[NDIM];           // Grid: physical range of grid
  int NProc[NDIM];              // Partitioning: process grid
  double Nu;                    // Kernel Param: smoothness
  int LenTheta;                 // Kernel Param: number of parameters
  double Theta0[NDIM];          // Kernel Param: initial guess
  int NumRandVec;               // SAA: number of random vectors
  int NumLapFilter;             // Linear solver: number of filtering
  double CG_RTol;               // Linear solver: CG rtol
  int CG_MaxIt;                 // Linear solver: CG maxit
  bool bElimAllReduce;          // Linear solver: optimization for CG
  bool bNeedPartials[3];        // Nonlinear solver: Jacobian
  char DataFile[MAX_FLEN];      // Data file: file name

private:

  void CalNProc(void);
  void CalNumLapFilter(void);

  MPI_Comm Comm;

};


//-------------------------------------------------------------------------
// A class that wraps all data structures needed to evaluate f and jac
class ScoreCtx {

public:

  ScoreCtx(RegularGrid *_Grid, Matern *_Kernel,
           RegularGridCovarMat<Matern> *_CovarMat,
           ApproxScoreFunc<RegularGridData, RegularGridCovarMat<Matern> > *_S,
           int _LenTheta, int _NumLapFilter, bool *_bNeedPartials,
           double *_F, double **_J);

  RegularGrid *Grid;
  Matern *Kernel;
  RegularGridCovarMat<Matern> *CovarMat;
  ApproxScoreFunc<RegularGridData, RegularGridCovarMat<Matern> > *S;
  int LenTheta;
  int NumLapFilter;
  bool *bNeedPartials;
  double *F;
  double **J;

};

#endif

// Author: J. Chen
// $Date: 2013/03/28 15:50:43$
