#ifndef _DATA_
#define _DATA_
// Defines distributed data.
// This data stucture does not know the specific data partitioning.

#include <mpi.h>

//--------------------------------------------------------------------------
class Data {

public:

  // Constructor, destructor
  Data();
  Data(MPI_Comm &comm);
  Data(const Data &D);
  Data& operator= (const Data &D);
  ~Data();

  // Set
  void SetLen(long len);
  void SetVal(double  val);
  void SetVal(double  val, long len);
  void SetVal(double *val);
  void SetVal(double *val, long len);
  void SetSymBernoulli(void);
  void SetSymBernoulli(unsigned int seed);
  void SetSymBernoulli(long len);
  void SetSymBernoulli(long len, unsigned int seed);

  // Get
  long GetLen(void);
  double *GetValPtr(void);

  // Major computational component
  double InProd(Data &B);
  double Norm2(void);
  double RelDiff(Data &B);      // = |self-B| / |self|

protected:

  // Needed by constructor and destructor
  void Init(void);
  void DeepCopy(const Data &D);
  void DeAllocMem(void);

  long Len;
  double *Val;
  MPI_Comm Comm;

private:

};

#endif

// Author: J. Chen
// $Date: 2013/03/28 15:50:43$
