#ifndef _KRIGING_
#define _KRIGING_
// Kriging.

//-------------------------------------------------------------------------
// Simple Kriging:
//
//   prediction = c' * inv(K) * z
//   variance = var{0} - c' * inv(K) * c
//-------------------------------------------------------------------------
template<class TData, class CovarMat>
class Kriging {

public:

  // Constructor, destructor
  Kriging();
  Kriging(const Kriging &K);
  Kriging& operator= (const Kriging &K);
  ~Kriging();

  // Major computational component
  void SimpleKriging(TData &gp_sample, CovarMat &covar_mat, TData &covar_vec,
                     double *prediction, double *variance);

private:

  // Needed by constructor and destructor
  void DeepCopy(const Kriging &K);
  void DeAllocMem(void);

};

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
Kriging<TData, CovarMat>::
Kriging() {
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
Kriging<TData, CovarMat>::
Kriging(const Kriging &K) {
  DeepCopy(K);
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
void Kriging<TData, CovarMat>::
DeepCopy(const Kriging &K) {
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
Kriging<TData, CovarMat>&
Kriging<TData, CovarMat>::
operator= (const Kriging &K) {
  if (this == &K) {
    return *this;
  }
  DeAllocMem();
  DeepCopy(K);
  return *this;
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
void Kriging<TData, CovarMat>::
DeAllocMem(void) {
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
Kriging<TData, CovarMat>::
~Kriging() {
  DeAllocMem();
}

//--------------------------------------------------------------------------
template<class TData, class CovarMat>
void Kriging<TData, CovarMat>::
SimpleKriging(TData &gp_sample, CovarMat &covar_mat, TData &covar_vec,
              double *prediction, double *variance) {
  TData tmp = gp_sample;
  covar_mat.KSolve(covar_vec, tmp);
  *prediction = tmp.InProd(gp_sample);
  *variance = covar_mat.GetKernel().GetValues(0, NULL, NULL, true)
    - tmp.InProd(covar_vec);
}

#endif

// Author: J. Chen
// $Date: 2013/03/28 15:50:43$
