#include <math.h>
#include "Utility.hpp"

//--------------------------------------------------------------------------
void ExceptionHandler(std::string msg, MPI_Comm comm) {
  int rank;
  MPI_Comm_rank(comm, &rank);
  if (rank == 0) {
    std::cout << msg;
  }
}

//--------------------------------------------------------------------------
double square(double x) {
  return x*x;
}

//--------------------------------------------------------------------------
double besselk(double nu, double x) {
  double zr = x;
  double zi = 0;
  double fnu = fabs(nu);
  int kode = 1;
  int n = 1;
  double cyr[1];
  double cyi[1];
  int nz;
  int ierr;
  zbesk_(&zr, &zi, &fnu, &kode, &n, cyr, cyi, &nz, &ierr);
  if (ierr) {
    std::ostringstream s_nu, s_x, s_ierr;
    s_nu << nu;
    s_x << x;
    s_ierr << ierr;
    throw "besselk: Error occur! nu = " + s_nu.str() +
      ", x = " + s_x.str() + ", ierr = " + s_ierr.str() + "\n";
  }
  return cyr[0];
}

//--------------------------------------------------------------------------
void besselk2(double nu, double x, double *y, double *y1, double *y2) {
  double zr = x;
  double zi = 0;
  double fnu = fabs(nu);
  double fnus;
  int kode = 1;
  int n;
  double cyr[3];
  double cyi[3];
  int nz;
  int ierr;

  //             |    nu >= 2     |   2 > nu >= 1  |  1 > nu > 0  
  // ------------+----------------+----------------+--------------
  // y NULL NULL | nu             | nu             | nu           
  // y  y1  NULL | nu, nu-1       | nu, nu-1       | nu, 1-nu     
  // y  y1   y2  | nu, nu-1, nu-2 | nu, nu-1, 2-nu | nu, 1-nu, nu 

  if (y != NULL && y1 != NULL && y2 != NULL) {
    if (nu >= 2) {
      n = 3;
      fnus = fnu - 2.0;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y2 = cyr[0];
      *y1 = cyr[1];
      *y = cyr[2];
    }
    else if (nu >= 1) {
      n = 2;
      fnus = fnu - 1;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y1 = cyr[0];
      *y = cyr[1];
      n = 1;
      fnus = 2.0 - fnu;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y2 = cyr[0];
    }
    else {
      n = 1;
      fnus = fnu;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y2 = cyr[0];
      *y = cyr[0];
      n = 1;
      fnus = 1.0 - fnu;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y1 = cyr[0];
    }
  }
  else if (y != NULL && y1 != NULL && y2 == NULL) {
    if (nu >= 1) {
      n = 2;
      fnus = fnu - 1.0;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y1 = cyr[0];
      *y = cyr[1];
    }
    else {
      n = 1;
      fnus = fnu;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y = cyr[0];
      n = 1;
      fnus = 1.0 - fnu;
      zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
      *y1 = cyr[0];
    }
  }
  else if (y != NULL && y1 == NULL && y2 == NULL) {
    n = 1;
    fnus = fnu;
    zbesk_(&zr, &zi, &fnus, &kode, &n, cyr, cyi, &nz, &ierr);
    *y = cyr[0];
  }

  if (ierr) {
    std::ostringstream s_nu, s_x, s_ierr;
    s_nu << nu;
    s_x << x;
    s_ierr << ierr;
    throw "besselk: Error occur! nu = " + s_nu.str() +
      ", x = " + s_x.str() + ", ierr = " + s_ierr.str() + "\n";
  }
}

// Author: J. Chen
// $Date: 2013/03/28 15:50:43$
